# N.B. This file is included from package/*/Makefile -- thus the weird path
include ../../config.mk

# Paths to:
# sources
SRC_DIR		:= $(ROOT_DIR)/package
DL_DIR		:= $(ROOT_DIR)/dl

# cooking room
BUILD_DIR	:= $(ROOT_DIR)/build
# intermediate stuff (libraries)
STAGING_DIR	?= $(BUILD_DIR)/.stage
# extended path including intermediate tools
PATH		:= ${PATH}:$(STAGING_DIR)/usr/bin
# kernel build
KBUILD		:= $(BUILD_DIR)/linux-$(KERNEL_VERSION)

# future /usr
TARGET_DIR	:= $(ROOT_DIR)/.usr

# Compiler flags
# N.B. we want static binaries
CFLAGS		+= -I$(STAGING_DIR)/usr/include -L$(STAGING_DIR)/usr/lib -static --static
CPPFLAGS	+= $(CFLAGS)
LDFLAGS		+= $(CFLAGS)

# Default package settings
DIR		?= $(PKG)-$(VERSION)
SOURCE		:= $(notdir $(URL))
BUILD		?= $(BUILD_DIR)/$(DIR)
SOURCEFORGE_MIRROR := kent

# Implied dependencies
#DEPS		?= 

KEY_BIN		?= $(PKG)
COPY_BIN	?= $(KEY_BIN)
BUILD_BIN	?= $(COPY_BIN)
BINS		?= $(addprefix $(BUILD)/,$(BUILD_BIN))
STAGE_BIN	?= $(addprefix $(STAGING_DIR)/usr/bin/,$(notdir $(BUILD_BIN)))
TARGET_BIN	?= $(addprefix $(TARGET_DIR)/bin/,$(notdir $(BUILD_BIN)))

BIN_DIR		?= $(TARGET_DIR)/bin
ETC_DIR		?= $(TARGET_DIR)/etc/$(PKG)
LIB_DIR		?= $(TARGET_DIR)/lib/$(PKG)
MOD_DIR		?= $(TARGET_DIR)/lib/modules/$(KERNEL_VERSION)

#COPY_TO_BUILD	?=

CONFIG_ENV	+=
CONFIG_OPT	+=

MAKE_OPT	?=
STAGE_OPT	?= install

# routines to copy package stuff to usrfs
define collect
	if test -n "$(COPY_$(1))" ; then mkdir -p $($(1)_DIR) ; cp -drf $(COPY_$(1)) $($(1)_DIR) ; fi
endef

define copy
	echo '	COPY $(PKG)'
	( cd $(BUILD) ; $(call collect,BIN) ; $(call collect,ETC) ; $(call collect,LIB) ; $(call collect,MOD) )
	for I in copy* ; do \
		if test -d $$I ; then cp -a $$I/* $(TARGET_DIR) ; fi \
	done
	$(call copy-post)
endef

all: $(DEPS) $(STAGE_BIN)
	$(call stage-post)
ifneq ($(findstring $(notdir $(realpath .)),$(CONFIG-y)),)
	$(call copy)
endif

collect:
ifneq ($(findstring $(notdir $(realpath .)),$(CONFIG-y)),)
	$(call copy)
endif

ifneq ($(strip $(DEPS)),)
$(DEPS):
	echo '	DEPEND $(@F)'
	$(MAKE) -C $(SRC_DIR)/$@
endif

$(STAGE_BIN): $(BINS)
	echo '	STAGE $(PKG)'
	# install intermediate stuff
	mkdir -p $(STAGING_DIR)
	CFLAGS="$(CFLAGS)" LDFLAGS="$(LDFLAGS)" CPPFLAGS="$(CPPFLAGS)" \
		$(MAKE) DESTDIR=$(STAGING_DIR) INSTALLROOT=$(STAGING_DIR) -C $(BUILD) -s $(STAGE_OPT) >$(BUILD)/4 2>&1
	# remove shared library
	rm -f $(STAGING_DIR)/usr/lib/*.so*
	# revalidate library dependencies
	if ls $(STAGING_DIR)/usr/lib/*.la >/dev/null 2>&1; then \
		sed -i -r "s,(\W)/usr/lib/,\1$(STAGING_DIR)/usr/lib/,g" $(STAGING_DIR)/usr/lib/*.la ; \
	fi

$(BINS): $(BUILD)/.configured
	echo '	BUILD $(PKG)'
	CFLAGS="$(CFLAGS)" LDFLAGS="$(LDFLAGS)" CPPFLAGS="$(CPPFLAGS)" \
		$(MAKE) -C ${BUILD} -s $(MAKE_OPT) >/dev/null 2>${BUILD}/3

$(BUILD)/.configured: $(BUILD)/.patched
	echo '	CONFIG $(PKG)'
	# clean
	-make -C $(BUILD) clean >/dev/null 2>&1
	# preconfigured?
	if test -f .config; then \
		cp .config $(BUILD) ; \
		make -C $(BUILD) oldconfig ; \
	fi
	# have ./configure? -> use it
	if test -x $(BUILD)/configure; then \
		( cd $(BUILD) && \
		CFLAGS="$(CFLAGS)" \
		CPPFLAGS="$(CPPFLAGS)" \
		LDFLAGS="$(LDFLAGS)" \
		$(CONFIG_ENV) ./configure \
		--prefix=/usr \
		--bindir=/usr/bin \
		--sbindir=/usr/bin \
		--libdir=/usr/lib \
		--libexecdir=/usr/lib \
		--sysconfdir=/usr/etc \
		--datadir=/usr/lib \
		--localstatedir=/var \
		--mandir=/usr/man \
		--infodir=/usr/doc \
		--includedir=/usr/include \
		--disable-nls \
		--disable-rpath \
		--disable-shared --enable-static \
		$(CONFIG_OPT) \
		>1 2>2 \
		) ; \
	fi
	touch $@

$(BUILD)/.patched: $(BUILD)/.unpacked
	echo '	PATCH $(PKG)'
	find . -maxdepth 1 -name \*.patch | xargs cat | ( cd $(BUILD) ; patch -p1 >2 2>&1 )
	touch $@

$(BUILD)/.unpacked: $(addprefix $(DL_DIR)/,$(SOURCE))
	echo '	UNPACK $(PKG)'
	# unpack sources
	mkdir -p $(BUILD_DIR)
	for I in $(filter-out $(COPY_TO_BUILD),$^); do \
		#tar -C $(BUILD_DIR) -xpf $$I; \
		if test "x$(VERSION)y" = xsnapshoty || grep -q `sha1sum $$I` $(ROOT_DIR)/sums ; then \
			tar -C $(BUILD_DIR) -xpf $$I || ( echo '		SHA1SUM FAILED!'; exit 1 ); \
		fi ; \
	done
	# copy additional stuff (if any)
	if test -n "$(COPY_TO_BUILD)"; then cp $(COPY_TO_BUILD) $(BUILD); fi
	$(call unpack-post)
	touch $@

$(addprefix $(DL_DIR)/,$(SOURCE)):
	echo '	FETCH $(PKG)'
	mkdir -p $(DL_DIR)
	for I in $(URL); do http_proxy=$(http_proxy) ftp_proxy=$(ftp_proxy) wget -c -Y on -P $(DL_DIR) $$I; done
	touch -c $@

sources: $(addprefix $(DL_DIR)/,$(SOURCE))

.PHONY: all $(DEPS) copy stage sources
