%ifndef LANG_ASM
%define LANG_ASM

;
; IF-ELIF-ELSE-ENDIF
;
%macro IF 1.nolist
    %push IF
    j%-1 near %$ifnot
%endmacro

%macro ELSE 0.nolist
    %ifnctx IF
	%error "Must have 'IF' before 'ELSE'!"
    %endif
    %repl ELSE
    jmp %$ifend
    %$ifnot:
%endmacro

%macro EXITIF 0-1.nolist
    %ifnctx IF
	%ifnctx ELSE
	    %error "Must have 'IF' or 'ELSE' before 'EXITIF'!"
	%endif
    %endif
    %if %0 > 0
	j%+1 near %$ifend
    %else
	jmp %$ifend
    %endif
%endmacro

%macro ENDIF 0.nolist
    %ifctx IF
	%$ifnot:
	%$ifend:
	%pop
    %elifctx ELSE
	%$ifend:
	%pop
    %else
	%error "Must have 'IF' or 'ELSE' before 'ENDIF'!"
    %endif
%endmacro

;
; PUSH/POP
;
%macro PUSH 1-*.nolist
    %push PUSH
    %rep %0
	push %1
	%rotate 1
    %endrep
%endmacro

%macro POP 1-*.nolist
    %ifnctx PUSH
	%error "Must have 'PUSH' before 'POP'!"
    %endif
    %rep %0
	%rotate -1
	pop %1
    %endrep
    %pop
%endmacro

;
; function call helpers
;
%macro	PARAM 1
    %push PARAM
    %ifstr %1
	[section .data]
	%$str db %1, 0
	__SECT__
	push %$str
    %else
	push %1
    %endif
    %pop
%endmacro

%macro	MOV 2.nolist
    %push MOV
    %ifstr %2
	[section .data]
	%$str db %2, 0
	__SECT__
	mov %1, %$str
    %else
	mov %1, %2
    %endif
    %pop
%endmacro

%macro	LET 4.nolist
    %push LET
    MOV %2, %3
    j%+1 %%done
    MOV %2, %4
    %%done:
    %pop
%endmacro

%macro	CALLC 1-*.nolist
    %push CALLC
    %if %0 > 1
	%rep %0 - 1
	    %rotate -1
	    PARAM %1
	%endrep
	%rotate -1
    %endif
    call %1
    %if %0 > 1
	add esp, byte (%0 - 1)*4
    %endif
    %pop
%endmacro

%macro	CCALLC 2-*.nolist
    j%-1 %%skip
    %push CALLC
    %if %0 > 2
	%rep %0 - 2
	    %rotate -1
	    PARAM %1
	%endrep
	%rotate -2
    %endif
    call %2
    %if %0 > 2
	add esp, byte (%0 - 2)*4
    %endif
    %pop
%%skip:
%endmacro

%macro	CALL	1-*
    %push CALL
    %if %0 > 1
	%rep %0 - 1
	    %rotate 1
	    PARAM %1
	%endrep
	%rotate 1
    %endif
    call %1
    %pop
%endmacro

%macro	CCALL 2-*.nolist
    j%-1 %%skip
    %push CALL
    %rotate 1
    %if %0 > 2
	%rep %0 - 2
	    %rotate 1
	    PARAM %1
	%endrep
	%rotate 2
    %endif
    call %1
    %pop
%%skip:
%endmacro

%macro	PROCC 1-*.nolist
    %push PROCC
    global %1
    %1:
;;
;; enter cdecl code
;;
    %assign %$argn %0 - 1
    %if %$argn > 0
	push ebp
	mov ebp, esp
	%assign %$arg 8
;;	%1_cdecl equ 1
	%rotate 1
	%rep %0 - 1
	    %xdefine .%1 ebp+%$arg
	    %assign %$arg %$arg+4
	    %rotate 1
	%endrep
    %endif
%endmacro

%macro	PROC 1-*.nolist
    %push PROC
    global %1
    %1:
;;
;; enter stdcall code
;;
    %assign %$argn %0 - 1
    %if %$argn > 0
	push ebp
	mov ebp, esp
	%assign %$arg 8
;;	%1_cdecl equ 0
	%rep %0 - 1
	    %rotate -1
	    %xdefine .%1 ebp+%$arg
	    %assign %$arg %$arg+4
	%endrep
    %endif
%endmacro

%macro	VAR 1-*.nolist
    %ifnctx PROC
	%ifnctx PROCC
	    %error "Must have 'PROC' or 'PROCC' before 'VAR'!"
	%endif
    %endif
    %assign %$hasvars 1
    %assign %$arg 4
    %rep %0
	push byte 0
	%xdefine .%1 ebp-%$arg
	%assign %$arg %$arg+4
	%rotate 1
    %endrep
%endmacro

%macro	VAR8 1-*.nolist
    %ifnctx PROC
	%ifnctx PROCC
	    %error "Must have 'PROC' or 'PROCC' before 'VAR'!"
	%endif
    %endif
    %assign %$hasvars 1
    %assign %$arg 8
    %rep %0
	push byte 0
	push byte 0
	%xdefine .%1 ebp-%$arg
	%assign %$arg %$arg+8
	%rotate 1
    %endrep
%endmacro

%macro	EXITPROC 0-1.nolist
    %ifnctx PROC
	%error "Must have 'PROC' before 'EXITPROC'!"
    %endif
    %if %0 == 0
	jmp %$endproc
    %else
	j%+1 near %$endproc
    %endif
%endmacro

%macro	ENDPROC 0.nolist
    %ifnctx PROC
	%ifnctx PROCC
	    %error "Must have 'PROC' or 'PROCC' before 'ENDPROC'!"
	%endif
    %endif
    %$endproc:
    %ifctx PROC
	%if %$argn > 0
;; leave stdcall code
	    %ifdef %$hasvars
	    	mov esp, ebp
	    %endif
	    pop ebp
	    ret %$argn * 4
	%else
	    ret
	%endif
    %else
	%if %$argn > 0
;; leave cdecl code
	    %ifdef %$hasvars
		mov esp, ebp
	    %endif
	    pop ebp
	%endif
	ret
    %endif
    %pop
%endmacro

;
; LOOP-ENDLOOP
;
%macro	LOOP 1.nolist
    %push LOOP
    mov eax, %1
    %$loop:
    dec eax
    jl near %$endloop
    push eax
%endmacro

%macro	LOOP0 1.nolist
    %push LOOP0
    %xdefine %$var %1
    %$loop:
%endmacro

%macro	EXITLOOP 0-1.nolist
    %ifnctx LOOP
	%error "Must have 'LOOP' before 'EXITLOOP'!"
    %endif
    %if %0 == 0
	add	esp, byte 4
	jmp %$endloop
    %else
	pop	eax
	j%+1 %$endloop
    %endif
%endmacro

%macro	CONTINUE 0-1.nolist
    %ifnctx LOOP
	%ifnctx LOOP0
	    %error "Must have 'LOOP' or 'LOOP0' before 'CONTINUE'!"
	%endif
    %endif
    %if %0 == 0
	jmp %$continue
    %else
	j%+1 near %$continue
    %endif
%endmacro

%macro	ENDLOOP 0.nolist
    %ifnctx LOOP
	%error "Must have 'LOOP' before 'ENDLOOP'!"
    %endif
    %$continue:
    pop eax
    jmp %$loop
    %$endloop:
    %pop
%endmacro

%macro	ENDLOOP0 0.nolist
    %ifnctx LOOP0
	%error "Must have 'LOOP0' before 'ENDLOOP0'!"
    %endif
    %$continue:
    dec %$var
    jg %$loop
    %pop
%endmacro

;
; switch-case-endcase-endswitch
;
%macro	SWITCH 1.nolist
    %push SWITCH
    push %1
%endmacro

%macro	CASE 0-1.nolist
    %ifnctx SWITCH
	%error "Must have 'SWITCH' before 'CASE'!"
    %endif
    %push CASE
    %if %0 > 0
	cmp dword [esp], %1
	jne %$endcase
    %endif
%endmacro

%macro	ENDCASE 0.nolist
    %ifnctx CASE
	%error "Must have 'CASE' before 'ENDCASE'!"
    %endif
    %$endcase:
    %pop
    jmp %$endswitch
%endmacro

%macro	ENDSWITCH 0.nolist
    %ifnctx SWITCH
	%error "Must have 'SWITCH' before 'ENDSWITCH'!"
    %endif
    %$endswitch:
    add esp, byte 4
    %pop
%endmacro

;
; conditional mov instruction
;
%macro	CMOV 3.nolist
    j%-1 %%skip
    mov %2, %3
%%skip:
%endmacro

%macro	LIMIT 2.nolist
    cmp %1, %2
    jng %%skip
    mov %1, %2
%%skip:
%endmacro

;
; bulk memory operations
;
%macro	COPY 3
    mov	esi, %2
    mov edi, %1
    mov ecx, %3
    cld
    shr	ecx, 2
    rep movsd
%endmacro

%macro	ZERO 2.nolist
    mov edi, %1
    mov ecx, %2
    cld
    xor	eax, eax
    shr	ecx, 2
    rep stosd
%endmacro

;
; data declare
;
%macro	INTEGER 1-*.nolist
    %push INT
    [section .bss]
    %assign %$i 1
    %rep %0
	%if %$i <= %0
	%ifnum %2
	    %1 resd %2
	    %assign %$i %$i+1
	    %rotate 1
	%else
	    %1 resd 1
	%endif
	%assign %$i %$i+1
	%rotate 1
	%endif
    %endrep
    __SECT__
    %pop
%endmacro

%macro	DOUBLE	1-*.nolist
    %push DOUBLE
    [section .bss]
    %assign %$i 1
    %rep %0
	%if %$i <= %0
	%ifnum %2
	    %1 resq %2
	    %assign %$i %$i+1
	    %rotate 1
	%else
	    %1 resq 1
	%endif
	%assign %$i %$i+1
	%rotate 1
	%endif
    %endrep
    __SECT__
    %pop
%endmacro

;
; floating point stuff
;
%macro	FMOV	2.nolist
    %push FMOV
    [section .data]
    %$tmp dq %2
    __SECT__
    fld	qword [%$tmp]
    fstp qword %1
    %pop
%endmacro

;
; structure allocation
;
%macro	STRUCTURE	1.nolist
    %define %%name %1
    istruc %1
	at %1.???, ?? ??
    iend
%endmacro


;
;
;
;;
%macro	OUTTT	1-2 4.nolist
    pushad
    sys_write	1, %1, %2
    popad
%endmacro



PROC	__stop__
	ret
ENDPROC

%macro	BREAKPOINT	0
    call __stop__
%endmacro

%endif


